import os
import pytest
import numpy as np

from . import util
from numpy.f2py.crackfortran import crackfortran


class TestData(util.F2PyTest):
    sources = [util.getpath("tests", "src", "crackfortran", "data_stmts.f90")]

    # For gh-23276
    def test_data_stmts(self):
        assert self.module.cmplxdat.i == 2
        assert self.module.cmplxdat.j == 3
        assert self.module.cmplxdat.x == 1.5
        assert self.module.cmplxdat.y == 2.0
        assert self.module.cmplxdat.medium_ref_index == np.array(1.+0.j)
        assert np.all(self.module.cmplxdat.z == np.array([3.5, 7.0]))
        assert np.all(self.module.cmplxdat.my_array == np.array([ 1.+2.j, -3.+4.j]))
        assert np.all(self.module.cmplxdat.my_real_array == np.array([ 1., 2., 3.]))
        assert np.all(self.module.cmplxdat.ref_index_one == np.array([13.0 + 21.0j]))
        assert np.all(self.module.cmplxdat.ref_index_two == np.array([-30.0 + 43.0j]))

    def test_crackedlines(self):
        mod = crackfortran(self.sources)
        assert mod[0]['vars']['x']['='] == '1.5'
        assert mod[0]['vars']['y']['='] == '2.0'
        assert mod[0]['vars']['my_real_array']['='] == '(/1.0d0, 2.0d0, 3.0d0/)'
        assert mod[0]['vars']['ref_index_one']['='] == '(13.0d0, 21.0d0)'
        assert mod[0]['vars']['ref_index_two']['='] == '(-30.0d0, 43.0d0)'
        assert mod[0]['vars']['my_array']['='] == '(/(1.0d0, 2.0d0), (-3.0d0, 4.0d0)/)'
        assert mod[0]['vars']['z']['='] == '(/3.5,  7.0/)'



from numpy.lib import NumpyVersion

version = NumpyVersion("1.8.0")

version.vstring
version.version
version.major
version.minor
version.bugfix
version.pre_release
version.is_devversion

version == version
version != version
version < "1.8.0"
version <= version
version > version
version >= "1.8.0"



import sys
from typing import Any

import numpy as np
import numpy.typing as npt

if sys.version_info >= (3, 11):
    from typing import assert_type
else:
    from typing_extensions import assert_type

nditer_obj: np.nditer

assert_type(np.nditer([0, 1], flags=["c_index"]), np.nditer)
assert_type(np.nditer([0, 1], op_flags=[["readonly", "readonly"]]), np.nditer)
assert_type(np.nditer([0, 1], op_dtypes=np.int_), np.nditer)
assert_type(np.nditer([0, 1], order="C", casting="no"), np.nditer)

assert_type(nditer_obj.dtypes, tuple[np.dtype[Any], ...])
assert_type(nditer_obj.finished, bool)
assert_type(nditer_obj.has_delayed_bufalloc, bool)
assert_type(nditer_obj.has_index, bool)
assert_type(nditer_obj.has_multi_index, bool)
assert_type(nditer_obj.index, int)
assert_type(nditer_obj.iterationneedsapi, bool)
assert_type(nditer_obj.iterindex, int)
assert_type(nditer_obj.iterrange, tuple[int, ...])
assert_type(nditer_obj.itersize, int)
assert_type(nditer_obj.itviews, tuple[npt.NDArray[Any], ...])
assert_type(nditer_obj.multi_index, tuple[int, ...])
assert_type(nditer_obj.ndim, int)
assert_type(nditer_obj.nop, int)
assert_type(nditer_obj.operands, tuple[npt.NDArray[Any], ...])
assert_type(nditer_obj.shape, tuple[int, ...])
assert_type(nditer_obj.value, tuple[npt.NDArray[Any], ...])

assert_type(nditer_obj.close(), None)
assert_type(nditer_obj.copy(), np.nditer)
assert_type(nditer_obj.debug_print(), None)
assert_type(nditer_obj.enable_external_loop(), None)
assert_type(nditer_obj.iternext(), bool)
assert_type(nditer_obj.remove_axis(0), None)
assert_type(nditer_obj.remove_multi_index(), None)
assert_type(nditer_obj.reset(), None)

assert_type(len(nditer_obj), int)
assert_type(iter(nditer_obj), np.nditer)
assert_type(next(nditer_obj), tuple[npt.NDArray[Any], ...])
assert_type(nditer_obj.__copy__(), np.nditer)
with nditer_obj as f:
    assert_type(f, np.nditer)
assert_type(nditer_obj[0], npt.NDArray[Any])
assert_type(nditer_obj[:], tuple[npt.NDArray[Any], ...])
nditer_obj[0] = 0
nditer_obj[:] = [0, 1]



import sys
from typing import Any

import numpy as np
import numpy.typing as npt
from numpy.linalg.linalg import QRResult, EigResult, EighResult, SVDResult, SlogdetResult

if sys.version_info >= (3, 11):
    from typing import assert_type
else:
    from typing_extensions import assert_type

AR_i8: npt.NDArray[np.int64]
AR_f8: npt.NDArray[np.float64]
AR_c16: npt.NDArray[np.complex128]
AR_O: npt.NDArray[np.object_]
AR_m: npt.NDArray[np.timedelta64]
AR_S: npt.NDArray[np.str_]

assert_type(np.linalg.tensorsolve(AR_i8, AR_i8), npt.NDArray[np.float64])
assert_type(np.linalg.tensorsolve(AR_i8, AR_f8), npt.NDArray[np.floating[Any]])
assert_type(np.linalg.tensorsolve(AR_c16, AR_f8), npt.NDArray[np.complexfloating[Any, Any]])

assert_type(np.linalg.solve(AR_i8, AR_i8), npt.NDArray[np.float64])
assert_type(np.linalg.solve(AR_i8, AR_f8), npt.NDArray[np.floating[Any]])
assert_type(np.linalg.solve(AR_c16, AR_f8), npt.NDArray[np.complexfloating[Any, Any]])

assert_type(np.linalg.tensorinv(AR_i8), npt.NDArray[np.float64])
assert_type(np.linalg.tensorinv(AR_f8), npt.NDArray[np.floating[Any]])
assert_type(np.linalg.tensorinv(AR_c16), npt.NDArray[np.complexfloating[Any, Any]])

assert_type(np.linalg.inv(AR_i8), npt.NDArray[np.float64])
assert_type(np.linalg.inv(AR_f8), npt.NDArray[np.floating[Any]])
assert_type(np.linalg.inv(AR_c16), npt.NDArray[np.complexfloating[Any, Any]])

assert_type(np.linalg.matrix_power(AR_i8, -1), npt.NDArray[Any])
assert_type(np.linalg.matrix_power(AR_f8, 0), npt.NDArray[Any])
assert_type(np.linalg.matrix_power(AR_c16, 1), npt.NDArray[Any])
assert_type(np.linalg.matrix_power(AR_O, 2), npt.NDArray[Any])

assert_type(np.linalg.cholesky(AR_i8), npt.NDArray[np.float64])
assert_type(np.linalg.cholesky(AR_f8), npt.NDArray[np.floating[Any]])
assert_type(np.linalg.cholesky(AR_c16), npt.NDArray[np.complexfloating[Any, Any]])

assert_type(np.linalg.qr(AR_i8), QRResult)
assert_type(np.linalg.qr(AR_f8), QRResult)
assert_type(np.linalg.qr(AR_c16), QRResult)

assert_type(np.linalg.eigvals(AR_i8), npt.NDArray[np.float64] | npt.NDArray[np.complex128])
assert_type(np.linalg.eigvals(AR_f8), npt.NDArray[np.floating[Any]] | npt.NDArray[np.complexfloating[Any, Any]])
assert_type(np.linalg.eigvals(AR_c16), npt.NDArray[np.complexfloating[Any, Any]])

assert_type(np.linalg.eigvalsh(AR_i8), npt.NDArray[np.float64])
assert_type(np.linalg.eigvalsh(AR_f8), npt.NDArray[np.floating[Any]])
assert_type(np.linalg.eigvalsh(AR_c16), npt.NDArray[np.floating[Any]])

assert_type(np.linalg.eig(AR_i8), EigResult)
assert_type(np.linalg.eig(AR_f8), EigResult)
assert_type(np.linalg.eig(AR_c16), EigResult)

assert_type(np.linalg.eigh(AR_i8), EighResult)
assert_type(np.linalg.eigh(AR_f8), EighResult)
assert_type(np.linalg.eigh(AR_c16), EighResult)

assert_type(np.linalg.svd(AR_i8), SVDResult)
assert_type(np.linalg.svd(AR_f8), SVDResult)
assert_type(np.linalg.svd(AR_c16), SVDResult)
assert_type(np.linalg.svd(AR_i8, compute_uv=False), npt.NDArray[np.float64])
assert_type(np.linalg.svd(AR_f8, compute_uv=False), npt.NDArray[np.floating[Any]])
assert_type(np.linalg.svd(AR_c16, compute_uv=False), npt.NDArray[np.floating[Any]])

assert_type(np.linalg.cond(AR_i8), Any)
assert_type(np.linalg.cond(AR_f8), Any)
assert_type(np.linalg.cond(AR_c16), Any)

assert_type(np.linalg.matrix_rank(AR_i8), Any)
assert_type(np.linalg.matrix_rank(AR_f8), Any)
assert_type(np.linalg.matrix_rank(AR_c16), Any)

assert_type(np.linalg.pinv(AR_i8), npt.NDArray[np.float64])
assert_type(np.linalg.pinv(AR_f8), npt.NDArray[np.floating[Any]])
assert_type(np.linalg.pinv(AR_c16), npt.NDArray[np.complexfloating[Any, Any]])

assert_type(np.linalg.slogdet(AR_i8), SlogdetResult)
assert_type(np.linalg.slogdet(AR_f8), SlogdetResult)
assert_type(np.linalg.slogdet(AR_c16), SlogdetResult)

assert_type(np.linalg.det(AR_i8), Any)
assert_type(np.linalg.det(AR_f8), Any)
assert_type(np.linalg.det(AR_c16), Any)

assert_type(np.linalg.lstsq(AR_i8, AR_i8), tuple[npt.NDArray[np.float64], npt.NDArray[np.float64], np.int32, npt.NDArray[np.float64]])
assert_type(np.linalg.lstsq(AR_i8, AR_f8), tuple[npt.NDArray[np.floating[Any]], npt.NDArray[np.floating[Any]], np.int32, npt.NDArray[np.floating[Any]]])
assert_type(np.linalg.lstsq(AR_f8, AR_c16), tuple[npt.NDArray[np.complexfloating[Any, Any]], npt.NDArray[np.floating[Any]], np.int32, npt.NDArray[np.floating[Any]]])

assert_type(np.linalg.norm(AR_i8), np.floating[Any])
assert_type(np.linalg.norm(AR_f8), np.floating[Any])
assert_type(np.linalg.norm(AR_c16), np.floating[Any])
assert_type(np.linalg.norm(AR_S), np.floating[Any])
assert_type(np.linalg.norm(AR_f8, axis=0), Any)

assert_type(np.linalg.multi_dot([AR_i8, AR_i8]), Any)
assert_type(np.linalg.multi_dot([AR_i8, AR_f8]), Any)
assert_type(np.linalg.multi_dot([AR_f8, AR_c16]), Any)
assert_type(np.linalg.multi_dot([AR_O, AR_O]), Any)
assert_type(np.linalg.multi_dot([AR_m, AR_m]), Any)
